#include <avr/io.h>
#include <avr/interrupt.h>

#include <util/delay.h>

#include <stdlib.h>
#include <stdio.h>

#include "usart.h"

#define STATE_IDLE	0
#define STATE_WATERING	1
#define THRESHOLD	450



int main(void)
{
	unsigned int moist_value, i;
	unsigned int state = STATE_IDLE;
	char moist_str[10];


	DDRB &= ~(1 << PB4);
	PORTB |= (1 << PB4);


	/* Timer 0 */

	DDRB |= (1 << PB3);

	/* Phase Corect mode */
	TCCR0A |= (1 << WGM00);

	/* prescaler 0 */
	TCCR0B |= (1 << CS00);

	OCR0A = 150;


	/* USART */

    	USART0_init();


	/* ADC  - PA1*/

	ADMUX = 0;
	ADMUX |= (1 << MUX0);          // ADC1 - channel 1
	ADMUX |= (1 << REFS0);         // AVCC with external capacitor at AREF pin
	ADCSRA = 0;
	ADCSRA |= (1 << ADPS2) | (1 << ADPS1) | (1 << ADPS0); //set clock prescaler at 128
	ADCSRA |= (1 << ADEN);         // enable ADC

	ADCSRA |= (1 << ADSC);         //start conversion

	while (1) {
		moist_value = ADC;
		sprintf(moist_str, "%u", moist_value);
		USART0_print(moist_str);
		USART0_print("\n");

		switch(state) {
			case STATE_IDLE:
				if (moist_value > THRESHOLD) {
					USART0_print("start water\n");
					TCCR0A |= (1 << COM0A1);
					/* prescaler 256 */
					TCCR0B |= (1 << CS02);
					state = STATE_WATERING;
				}

				for (i = 0; i < 20; i++) {
					if ((PINB & (1 << PB4)) == 0) {
						USART0_print("start button water\n");
						TCCR0A |= (1 << COM0A1);
						/* prescaler 256 */
						TCCR0B |= (1 << CS02);
						while ((PINB & (1 << PB4)) == 0)
							_delay_ms(1000);
						TCCR0B &= ~(1 << CS02);
						TCCR0B |= (1 << CS00);
						TCCR0A &= ~(1 << COM0A1);
						PORTB &= ~(1 << PB3);
						USART0_print("start button water\n");
						break;
					}
					_delay_ms(500);
				}
				break;

			case STATE_WATERING:
				if (moist_value < THRESHOLD) {
					USART0_print("stop water\n");
					TCCR0B &= ~(1 << CS02);
					TCCR0B |= (1 << CS00);
					TCCR0A &= ~(1 << COM0A1);
					PORTB &= ~(1 << PB3);
					state = STATE_IDLE;
				}
				_delay_ms(1000);
				break;

		}
		ADCSRA |= (1 << ADSC);         //start conversion
	}

    return 0;
}

